/*
 * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
 */
#ifndef __E_MOD_MAIN_H__
#define __E_MOD_MAIN_H__


#define SEARCH_REFRESH 10
#define MAX_SONG_LENGTH 255

#define PLAYER_XMMS 0
#define PLAYER_XMMS2 1
#define PLAYER_BMP 2
#define PLAYER_BMPX 3
#define PLAYER_AUDACIOUS 4
#define PLAYER_ECLAIR 5
#define PLAYER_AMAROK 6
#define PLAYER_MPD 7

#define THEME_DEFAULT "winter"

#include "e.h"
#include <glib-1.2/glib.h>
#include <iconv.h>
#include "config.h"

typedef struct _Config		Config;
typedef struct _Config_Face	Config_Face;
typedef struct _Eloquence	Eloquence;

struct _Config
{
   Evas_List *faces;
};

struct _Config_Face
{
   unsigned char enabled;
   char*	 theme;
   unsigned char player;
   double        poll_time;
};

struct _Eloquence
{
   Evas_List   *faces;
   E_Menu      *config_menu;
   Config      *conf;
};

typedef struct _Eloquence_Face
{
   Eloquence       *eloquence;
   Evas            *evas;
   Evas_Object     *eloquence_object;
   Evas_Object     *event_object;
   Evas_Coord       xx, yy, ww;
   E_Menu          *menu_bg;
   E_Menu          *menu_trans;
   E_Menu          *menu_font;
   E_Config_DD     *conf_edd;
					  
   E_Container *con;
   E_Menu      *menu;
   E_Menu      *theme_menu;
   E_Menu      *player_menu;
   E_Menu      *poll_menu;
   Config_Face *conf;

  /* Evas Objects */
   E_Gadman_Client *gmc;

   /* Specific Objects */
   Ecore_Timer	*update_timer;
   Ecore_Timer	*recheck_timer;
   gchar	*song;
   gchar	outbuf[MAX_SONG_LENGTH];
   gchar	*song_utf8;
   iconv_t	cd;
   union {
      gint	id;
      void	*ptr;
   } session;
#ifdef HAVE_AMAROK
   gchar	outbuf2[MAX_SONG_LENGTH];
   char		amarok_temp_isPlaying[1024];
   char		amarok_temp_nowPlaying[MAX_SONG_LENGTH];
   char		amarok_com_isPlaying[1024];
   char		amarok_com_nowPlaying[1024];
#endif
} Eloquence_Face;

EAPI void *e_modapi_init     (E_Module *module);
EAPI int   e_modapi_shutdown (E_Module *module);
EAPI int   e_modapi_save     (E_Module *module);
EAPI int   e_modapi_info     (E_Module *module);
EAPI int   e_modapi_about    (E_Module *module);

void draw_text(void *data);

/* xmms.c */
#ifdef HAVE_XMMS_XMMSCTRL_H
int xmms_update_song(void *data);
void xmms_control_play_pause(void *data);
void xmms_control_track_next(void *data);
void xmms_control_track_prev(void *data);
void xmms_control_vol_inc(void *data);
void xmms_control_vol_dec(void *data);
void xmms_control_shuffle(void *data);
#endif

/* bmp.c */
#ifdef HAVE_BMP_BEEPCTRL_H
int bmp_update_song(void *data);
void bmp_control_play_pause(void *data);
void bmp_control_track_next(void *data);
void bmp_control_track_prev(void *data);
void bmp_control_vol_inc(void *data);
void bmp_control_vol_dec(void *data);
void bmp_control_shuffle(void *data);
#endif

/* audacious.c */
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
int audacious_update_song(void *data);
void audacious_control_play_pause(void *data);
void audacious_control_track_next(void *data);
void audacious_control_track_prev(void *data);
void audacious_control_vol_inc(void *data);
void audacious_control_vol_dec(void *data);
void audacious_control_shuffle(void *data);
#endif

/* bmpx.c */
#ifdef HAVE_BMPX_METADATA_H
int bmpx_update_song(void *data);
void bmpx_control_play_pause(void *data);
void bmpx_control_track_next(void *data);
void bmpx_control_track_prev(void *data);
void bmpx_control_vol_inc(void *data);
void bmpx_control_vol_dec(void *data);
void bmpx_control_shuffle(void *data);
#endif

/* amarok.c */
#ifdef HAVE_AMAROK
void amarok_init(void *data);
int amarok_update_song(void *data);
void amarok_control_play_pause(void *data);
void amarok_control_track_next(void *data);
void amarok_control_track_prev(void *data);
void amarok_control_vol_inc(void *data);
void amarok_control_vol_dec(void *data);
void amarok_control_shuffle(void *data);
#endif

/* mpd.c */
#ifdef HAVE_LIBMPD_0_01_LIBMPD_LIBMPD_H
void mpd_connect(Eloquence_Face *face);
void mpd_disconnect(Eloquence_Face *face);
int mpd_update_song(void *data);
void mpd_control_play_pause(void *data);
void mpd_control_track_next(void *data);
void mpd_control_track_prev(void *data);
void mpd_control_vol_inc(void *data);
void mpd_control_vol_dec(void *data);
void mpd_control_shuffle(void *data);
#endif

#endif /* __E_MOD_MAIN_H__ */
